//=============================================================================
// IPLogger.
//
// Copyright(c) 2003, Winged Unicorn.
//=============================================================================
class IPLogger expands ServerActor
	config(MCoopServerTools);


struct Entry_Type
{
	var() config string IPAddress;
	var() config string PlayerName;
};

var(IPLogger) globalconfig Entry_Type BannedIPs[50];
var(IPLogger) globalconfig Entry_Type TempBannedIPs[20];
var(IPLogger) globalconfig Entry_Type LoggedIPs[250];

// turn this to False if you want to use an external IP tool
var(IPLogger) globalconfig bool bEnabled;

// run time
var bool bInitialized;


function PreBeginPlay()
{
	if ( !bEnabled || bInitialized )
		return;

	bInitialized = True;
	InitArrays();
}

function InitArrays()
{
	local int i, target;
	local Entry_Type B_IPs[ArrayCount(BannedIPs)];
	local Entry_Type L_IPs[ArrayCount(LoggedIPs)];

	// save ban list in a temp one and clear it
	target = 0;
	for ( i=0; i<ArrayCount(BannedIPs); i++ )
	{
		if ( BannedIPs[i].IPAddress != "" )
		{
			B_IPs[target] = BannedIPs[i];
			target++;
		}
		BannedIPs[i].IPAddress  = "";
		BannedIPs[i].PlayerName = "";
	}

	// load the new ban list
	for ( i=0; i<target; i++ )
		BannedIPs[i] = B_IPs[i];

	// save log list in a temp one and clear it
	target = 0;
	for ( i=0; i<ArrayCount(LoggedIPs); i++ )
	{
		if ( LoggedIPs[i].IPAddress != "" )
		{
			L_IPs[target] = LoggedIPs[i];
			target++;
		}
		LoggedIPs[i].IPAddress  = "";
		LoggedIPs[i].PlayerName = "";
	}

	// load the new log list
	for ( i=0; i<target; i++ )
		LoggedIPs[i] = L_IPs[i];

	// clear temp ban list
	for ( i=0; i<ArrayCount(TempBannedIPs); i++ )
	{
		TempBannedIPs[i].IPAddress  = "";
		TempBannedIPs[i].PlayerName = "";
	}

	SaveConfig();
}

// called at PreLogin, it checks if the given IP is banned or not
// if it is, the function also records the "new" name of the bugger
function bool CheckBanIP(string InName, string IPAddress)
{
	local int i;
	local string FixedName, Result;

	FixedName = class'MCoopDLL'.static.CheckString(InName);

	for (i=0; i<ArrayCount(BannedIPs) && (BannedIPs[i].IPAddress != ""); i++)
	{
		if ( CheckIP(IPAddress, BannedIPs[i].IPAddress) )
		{
			if ((BannedIPs[i].PlayerName == "") || (BannedIPs[i].PlayerName == "<< Unknown >>"))
			{
				BannedIPs[i].PlayerName = FixedName;
				SaveConfig();
			}
			else if ( InStr( BannedIPs[i].PlayerName, FixedName ) == -1 )
			{
				BannedIPs[i].PlayerName = BannedIPs[i].PlayerName $", "$ FixedName;
				SaveConfig();
			}

			Result = InName $" ["$ IPAddress $"] was blocked because he's banned from this server ;)";
			AdminMsg(Result);
			IPLog(Result);

			return True;
		}
	}

	return False;
}

// also called at PreLogin, it checks if the given IP is temp banned or not
// if it is, the function also records the "new" name of the bugger
function bool CheckTempBanIP(string InName, string IPAddress)
{
	local int i;
	local string Result;

	for ( i=0; i<ArrayCount(TempBannedIPs) && (TempBannedIPs[i].IPAddress != ""); i++ )
	{
		if ( TempBannedIPs[i].IPAddress == IPAddress )
		{
			if ( TempBannedIPs[i].PlayerName == "<< Unknown >>" )
			{
				TempBannedIPs[i].PlayerName = InName;
				SaveConfig();
			}
			else if ( InStr( TempBannedIPs[i].PlayerName, InName ) == -1 )
			{
				TempBannedIPs[i].PlayerName = TempBannedIPs[i].PlayerName $ ", " $ InName;
				SaveConfig();
			}

			Result = InName $"["$ IPAddress $"] was blocked because he's temporary banned from this server ;)";
			AdminMsg(Result);
			IPLog(Result);

			return True;
		}
	}

	return False;
}

function LogIP(string PlayerName, string IPAddress)
{
	local string FixedName;
	local int i;

	if ((IPAddress == "") || (Left(IPAddress, 1) == "0"))
		return;

	FixedName = class'MCoopDLL'.static.CheckString(PlayerName);

	for ( i=0; i<ArrayCount(LoggedIPs); i++)
	{
		if (i == (ArrayCount(LoggedIPs) - 1))
		{
			IPLog("WARNING, IPLogger is FULL! It will now auto-clean the oldest entries...");
			CleanLog();
			LogIP(PlayerName, IPAddress);
		}
		if ( LoggedIPs[i].IPAddress == IPAddress )
		{
			if ( InStr( LoggedIPs[i].PlayerName, FixedName ) == -1 )
			{
				if ( LoggedIPs[i].PlayerName == "" )
					LoggedIPs[i].PlayerName = FixedName;
				else
					LoggedIPs[i].PlayerName = LoggedIPs[i].PlayerName $", "$ FixedName;
			}
			SaveConfig();
			break;
		}
		if ( LoggedIPs[i].IPAddress == "" )
		{
			LoggedIPs[i].IPAddress  = IPAddress;
			LoggedIPs[i].PlayerName = FixedName;

			SaveConfig();
			break;
		}
	}
}

function CleanLog()
{
	local int i;

	for (i=0; i<50; i++)
	{
		LoggedIPs[i].IPAddress  = "";
		LoggedIPs[i].PlayerName = "";
	}

	IPLog("Logged IPs list has been cleaned ;)");
}

// add a IP to ban list
function string BanIP(string IPAddress, string PlayerName)
{
	local string FixedName, Output;
	local int i;

	if ( Len(IPAddress) < 8 )
		Output = "The IP you entered is not correct... :|";
	else
	{
		if (PlayerName != "<< Unknown >>")
			FixedName = class'MCoopDLL'.static.CheckString(PlayerName);

		for ( i=0; i<ArrayCount(BannedIPs); i++ )
		{
			if ( BannedIPs[i].IPAddress == IPAddress )
			{
				Output = "This IP is already banned :P";
				if ( BannedIPs[i].PlayerName == "<< Unknown >>" )
					BannedIPs[i].PlayerName = FixedName;
				else if ( InStr( BannedIPs[i].PlayerName, FixedName) == -1 )
					BannedIPs[i].PlayerName = BannedIPs[i].PlayerName $", "$ FixedName;
				break;
			}
			else if ( BannedIPs[i].IPAddress == "" )
			{
				BannedIPs[i].IPAddress  = IPAddress;
				BannedIPs[i].PlayerName = FixedName;
				SaveConfig();

				if ( PlayerName != "" )
					Output = PlayerName $" ["$ IPAddress $"] has been successfully banned in slot "$ i;
				else
					Output = "The IP Address ["$ IPAddress $"] has been added to banlist in slot "$ i;

				break;
			}
		}
	}

	if ( Output == "" )
		Output = "WARNING! No more ban spots left!!";

	IPLog(Output);
	return Output;
}

// temporary ban for players who reconnected too quickly
// (it lasts only until map changes)
function string TempBanIP(string PlayerIP, string PlayerName)
{
	local int i;
	local string Output;

	if ( Len(PlayerIP) < 8 )
		Output = "The IP you entered is not correct... :|";
	else
	{
		for ( i=0; i<ArrayCount(TempBannedIPs); i++ )
		{
			if ( TempBannedIPs[i].IPAddress == PlayerIP )
			{
				Output = "This IP is already banned :P";
				if ( TempBannedIPs[i].PlayerName == "<< Unknown >>" )
					TempBannedIPs[i].PlayerName = PlayerName;
				else if ( InStr( TempBannedIPs[i].PlayerName, PlayerName) == -1 )
					TempBannedIPs[i].PlayerName = TempBannedIPs[i].PlayerName $", "$ PlayerName;

				break;
			}
			else if ( TempBannedIPs[i].IPAddress == "" )
			{
				TempBannedIPs[i].IPAddress  = PlayerIP;
				TempBannedIPs[i].PlayerName = PlayerName;
				SaveConfig();

				if ( PlayerName != "" )
					Output = PlayerName $" ["$ PlayerIP $"] has been successfully banned in slot "$ i;
				else
					Output = "The IP Address ["$ PlayerIP $"] has been added to banlist in slot "$ i;

				break;
			}
		}
	}

	if ( Output == "" )
		Output = "WARNING! No more temporary ban spots left!!";

	IPLog(Output);
	return Output;
}

// remove the ban from a given slot
function string UnBanIP(int Slot)
{
	local string Output;

	if ( (Slot < ArrayCount(BannedIPs)) && (Slot >= 0) && (BannedIPs[Slot].IPAddress != "") )
	{
		Output = BannedIPs[Slot].PlayerName $" ["$ BannedIPs[Slot].IPAddress $"] in slot "$ Slot $" has been successfully unbanned";
		IPLog( Output );
		BannedIPs[Slot].IPAddress  = "";
		BannedIPs[Slot].PlayerName = "";
		SaveConfig();
	}
	else
		Output = "Slot not found :/";

	IPLog(Output);
	return Output;
}

function ShowBannedIPs(MCoopUnrealIPlayer Requester)
{
	local int i;

	if ( !Requester.bAdmin )
		return;

	Requester.ClientMessage("    (: *** IPLogger: BANNED IPs *** :) ");
	Requester.ClientMessage("");

	for ( i=0; i<ArrayCount(BannedIPs) && (BannedIPs[i].IPAddress != ""); i++ )
		Requester.ClientMessage(" Slot: "$ i @"   Name: "$ BannedIPs[i].PlayerName @"   IP: "$ BannedIPs[i].IPAddress);

	if ( i == 0 )
		Requester.ClientMessage("       - No IPs banned :) -");
	Requester.ClientMessage(" ");
}

// return all the logged names of a certain IP
function string WhoIs(PlayerReplicationInfo PRI)
{
	local string IPAddress;
	local int i;

	IPAddress = MCoopPlayerReplicationInfo(PRI).PlayerIP;
	for (i=0; i<ArrayCount(LoggedIPs) && (LoggedIPs[i].IPAddress != ""); i++)
		if ( LoggedIPs[i].IPAddress == IPAddress )
	 		return LoggedIPs[i].PlayerName;
}

// this function returns True if the given IPAddress is supposed
// to be a private address. Note:
//
// 127.0.0.0 - 127.255.255.255		loopback
// 10.0.0.0 - 10.255.255.255		class A
// 172.16.0.0 - 172.31.255.255		class B
// 192.168.0.0 - 192.168.255.255	class C
//
function bool IsALanPlayer(string IPAddress)
{
	if ( (Left(IPAddress, 4) ~= "127.")
		|| (Left(IPAddress, 7) ~= "192.168")
		|| (Left(IPAddress, 3) ~= "10.")
		|| CheckIP(IPAddress, "172.16.0.0-172.31.255.255") )
		return True;

	return False;
}

// check if the given IP is banned (also supports ranges)
// original function by BOZO (IPLogger 0.3b) - all credits to him!!
function bool CheckIP(string PlayerIP, string BannedIP)
{
	local bool Result;
	local int Offset, BeginR, EndR, Check;
	local string BeginRange, EndRange;

	Offset = InStr(BannedIP, "-");

	if (Offset == -1)
	{
		if (PlayerIP == BannedIP)
			Result = True;
	}

	if (Offset != -1)
	{
		BeginRange = Left(BannedIP, Offset);
		EndRange = Mid(BannedIP, Offset + 1, Len(BannedIP));
		BeginR = IPStringToInt(BeginRange);
		EndR   = IPStringToInt(EndRange);
		Check  = IPStringToInt(PlayerIP);

		if ((Check >= BeginR) && (Check <= EndR))
			Result = True;
	}

	return Result;
}

// convert IP to int
// original function by BOZO (IPLogger 0.3b) - all credits to him!!
final function int IPStringToInt(string IPString)
{
	local int Offset;
	local int A, B, C, D;

	Offset = InStr(IPString, ".");
	A = int( Left(IPString, Offset) );
	IPString = Mid(IPString, Offset + 1, Len(IPString));

	Offset = InStr(IPString, ".");
	B = int( Left(IPString, Offset) );
	IPString = Mid(IPString, Offset + 1, Len(IPString));

	Offset = InStr(IPString, ".");
	C = int( Left(IPString, Offset) );
	IPString = Mid(IPString, Offset + 1, Len(IPString));

	D = int(IPString);

	return ( (A*256*256*256) + (B*256*256) + (C*256) + D );
}

function IPLog(string Msg)
{
	Log("** "$ Msg $" **", 'IPLogger');
}

function AdminMsg(coerce string Msg)
{
	local Pawn aPawn;

	for ( aPawn=Level.PawnList; aPawn!=None; aPawn=aPawn.nextPawn )
		if ( aPawn.bIsPlayer && (MCoopUnrealIPlayer(aPawn) != None) && (PlayerPawn(aPawn).bAdmin || MCoopUnrealIPlayer(aPawn).bAdminChild) )
			PlayerPawn(aPawn).ClientMessage("** MCoop: "@ Msg @" **", 'Pickup', True);
}

defaultproperties
{
				bEnabled=True
}
